@@ -1,4 +1,4 @@
-/* $Id: strings.cpp 19223 2010-02-23 23:26:37Z rubidium $ */
+/* $Id: strings.cpp 19926 2010-06-04 21:07:08Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -231,9 +231,9 @@ static char *FormatZerofillNumber(char *buff, int64 number, int64 count, const c
 	return FormatNumber(buff, number, last, "", 20 - count);
 }
 
-static char *FormatHexNumber(char *buff, int64 number, const char *last)
+static char *FormatHexNumber(char *buff, uint64 number, const char *last)
 {
-	return buff + seprintf(buff, last, "0x%x", (uint32)number);
+	return buff + seprintf(buff, last, "0x" OTTD_PRINTFHEX64, number);
 }
 
 /**
@@ -390,17 +390,17 @@ static int DeterminePluralForm(int64 count)
 		case 2:
 			return n > 1;
 
-		/* Three forms, special case for zero
+		/* Three forms, special case for 0 and ending in 1, except those ending in 11
 		 * Used in:
 		 *   Latvian */
 		case 3:
 			return n % 10 == 1 && n % 100 != 11 ? 0 : n != 0 ? 1 : 2;
 
-		/* Three forms, special case for one and two
+		/* Five forms, special case for one, two, 3 to 6 and 7 to 10
 		 * Used in:
 		 *   Gaelige (Irish) */
 		case 4:
-			return n == 1 ? 0 : n == 2 ? 1 : 2;
+			return n == 1 ? 0 : n == 2 ? 1 : n < 7 ? 2 : n < 11 ? 3 : 4;
 
 		/* Three forms, special case for numbers ending in 1[2-9]
 		 * Used in:
@@ -410,7 +410,7 @@ static int DeterminePluralForm(int64 count)
 
 		/* Three forms, special cases for numbers ending in 1 and 2, 3, 4, except those ending in 1[1-4]
 		 * Used in:
-		 *   Croatian, Russian, Slovak, Ukrainian */
+		 *   Croatian, Russian, Ukrainian */
 		case 6:
 			return n % 10 == 1 && n % 100 != 11 ? 0 : n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 >= 20) ? 1 : 2;
 
@@ -434,7 +434,7 @@ static int DeterminePluralForm(int64 count)
 
 		/* Three forms, special cases for one and 2, 3, or 4
 		 * Used in:
-		 *   Czech */
+		 *   Czech, Slovak */
 		case 10:
 			return n == 1 ? 0 : n >= 2 && n <= 4 ? 1 : 2;
 
@@ -463,6 +463,12 @@ static int DeterminePluralForm(int64 count)
 				default:
 					NOT_REACHED();
 			}
+
+		/* Four forms: one, 0 and everything ending in 02..10, everything ending in 11..19.
+		 * Used in:
+		 *  Maltese */
+		case 12:
+			return (n == 1 ? 0 : n == 0 || (n % 100 > 1 && n % 100 < 11) ? 1 : (n % 100 > 10 && n % 100 < 20) ? 2 : 3);
 	}
 }
 
@@ -552,11 +558,12 @@ static char *FormatString(char *buff, const char *str, int64 *argv, uint casei,
 	WChar b;
 	int64 *argv_orig = argv;
 	uint modifier = 0;
+	char *buf_start = buff;
 
 	while ((b = Utf8Consume(&str)) != '\0') {
 		if (SCC_NEWGRF_FIRST <= b && b <= SCC_NEWGRF_LAST) {
 			/* We need to pass some stuff as it might be modified; oh boy. */
-			b = RemapNewGRFStringControlCode(b, &buff, &str, argv);
+			b = RemapNewGRFStringControlCode(b, buf_start, &buff, &str, argv);
 			if (b == 0) continue;
 		}
 
@@ -856,7 +863,7 @@ static char *FormatString(char *buff, const char *str, int64 *argv, uint casei,
 			} break;
 
 			case SCC_HEX: // {HEX}
-				buff = FormatHexNumber(buff, GetInt64(&argv), last);
+				buff = FormatHexNumber(buff, (uint64)GetInt64(&argv), last);
 				break;
 
 			case SCC_BYTES: // {BYTES}
@@ -957,7 +964,7 @@ static char *FormatString(char *buff, const char *str, int64 *argv, uint casei,
 
 				assert(e != NULL);
 
-				if (e->name != NULL) {
+				if (e->name != NULL && e->info.string_id != STR_NEWGRF_INVALID_ENGINE) {
 					buff = strecpy(buff, e->name, last);
 				} else {
 					buff = GetStringWithArgs(buff, e->info.string_id, NULL, last);
